package org.zeroturnaround.javarebel.integration.spring;

/**
 * A <tt>boolean</tt> value that may be updated atomically.
 * 
 * @author Rein Raudjärv
 */
public class AtomicBoolean {

  private boolean value;

  /**
   * Creates a new <tt>AtomicBoolean</tt> with the given initial value.
   *
   * @param initialValue the initial value
   */
  public AtomicBoolean(boolean initialValue) {
    this.value = initialValue;
  }

  /**
   * Returns the current value.
   *
   * @return the current value
   */
  public synchronized boolean get() {
    return value;
  }

  /**
   * Unconditionally sets to the given value.
   *
   * @param newValue the new value
   */
  public synchronized void set(boolean value) {
    this.value = value;
  }

  /**
   * Atomically set the value to the given updated value
   * if the current value <tt>==</tt> the expected value.
   * @param expect the expected value
   * @param update the new value
   * @return true if successful. False return indicates that
   * the actual value was not equal to the expected value.
   */
  public synchronized boolean compareAndSet(boolean expect, boolean update) {
    if (value == expect) {
      value = update;
      return true;
    }
    return false; 
  }

}
